<?php if (!defined('BASEPATH')) die('No direct script access allowed');

/**
 * fieldtype
 *
 * @package     MSM Site Select fieldtype module
 * @category    Modules
 * @author      Rein de Vries <info@reinos.nl>
 * @link        http://reinos.nl
 * @copyright   Copyright (c) 2015 Reinos.nl Internet Media
 */

include(PATH_THIRD.'url_select/config.php');


class Url_select_ft extends EE_Fieldtype
{

    public $info = array(
        'name'      => URL_SELECT_NAME,
        'version'   => URL_SELECT_VERSION
    );

    public $settings = array();

    public $default_settings = array();

    public $has_array_data = TRUE;

    private $prefix;

    // ----------------------------------------------------------------------

    /**
     * Constructor
     *
     * @access public
     *
     * Calls the parent constructor
     */
    public function __construct()
    {
        if (version_compare(APP_VER, '2.1.4', '>')) { parent::__construct(); } else { parent::EE_Fieldtype(); }

        $this->prefix = URL_SELECT_MAP.'_';

        //load lang file
        ee()->lang->loadfile(URL_SELECT_MAP);

        //require settings
        require 'settings.php';
    }

    // ---------------------------------------------------------------------- 
    // Before saving the content to the database
    // ---------------------------------------------------------------------- 

    /**
     * save (Native EE)
     *
     * @access public
     */
    public function save($data)
    {
        return $this->_save($data);
    }

    // ----------------------------------------------------------------------

    /**
     * save
     *
     * @access public
     */
    private function _save($data = '')
    {
        if($data['entry_id'] != 'custom')
        {
            $data['custom'] = '';
        }

        return str_replace('&', '|', http_build_query($data));
    }

    // ----------------------------------------------------------------------
    // Display the field for all types
    // ----------------------------------------------------------------------

    /**
     * display_field
     *
     * @access public
     */
    function display_field($data)
    {
        //generate the data
        return $this->_display_field($data);
    }

    // ----------------------------------------------------------------------

    /**
     * display_field
     *
     * @access public
     */
    private function _display_field($data, $type = 'default')
    {
        //load the js
        ee()->cp->load_package_js('url_select');

        //format the data
        $data = $this->format_data($data);

        //fieldname
        switch($type)
        {
            case 'matrix': $field_name = $this->cell_name ;
                break;
            case 'default':
            default : $field_name = $this->field_name;
                break;
        }

        //get the sitepages
        $site_pages = $this->get_site_pages();

        //no structure, shame on you
        if($site_pages === false)
            return 'Structure is needed to use the fieldtype';

        //add custom url
        if(isset($site_pages['uris']))
        {
            $site_pages = $this->add_zero($site_pages['uris']);

            $site_pages['custom'] = '--- '.lang('Custom url').' ---';
        }

        //return data to display
        return form_dropdown($this->field_name.'[entry_id]', $site_pages, $data['entry_id'], 'class="url_select" style="width: 45%;    padding: 3px; margin-right: 5px;"').
             form_input($this->field_name.'[custom]', $data['custom'], 'placeholder="'.('http://').'" class="url_select_custom" style="display:none;margin-top:5px;width: 45%;"');

    }

    // ----------------------------------------------------------------------

    /**
     * replace_tag
     *
     * @access public
     */
    public function replace_tag($data, $params = array(), $tagdata = FALSE)
    {
        //format the data
        $data = $this->format_data($data);

        $return = '';

        if($data['entry_id'] != 'custom' && $data['entry_id'] != '' && $data['entry_id'] != '0')
        {
            $return = $this->get_site_pages($data['entry_id']);
        }
        else
        {
            $return = $data['custom'];
        }

        return $return;
    }

    // ----------------------------------------------------------------------

    /**
     * replace_tag_catchall
     *
     * @access public
     */
    function replace_tag_catchall($data, $params = array(), $tagdata = FALSE, $modifier)
    {
        //format the data
        $data = $this->format_data($data);

        if($modifier == 'custom')
        {
            return $data['custom'];
        }
    }

    // ----------------------------------------------------------------------
    // Display the settings for all types
    // ----------------------------------------------------------------------

    /**
     * Display settings screen (Default EE)
     *
     * @access  public
     */
    function display_settings($data)
    {
        foreach($this->_display_settings($data) as $val)
        {
            ee()->table->add_row($val);
        }
    }

    // --------------------------------------------------------------------

    /**
     * Display settings screen
     *
     * @access  public
     */
    private function _display_settings($data, $options = array(), $type = '')
    {
        $matrix_input = isset($options['matrix_input']) ? $options['matrix_input'] : '';

        $return = array();

        if(!empty($this->fieldtype_settings))
        {
            foreach($this->fieldtype_settings as $field=>$options)
            {
                //set the override value
                $override_value = false;
                if(isset($options['override']))
                {
                    $override_value = ee()->gmaps_settings->get_setting($options['override'].'_override');
                }

                //is overrided?
                if($override_value !== false)
                {
                    $value = $override_value;

                }
                //default value
                else
                {
                    $value = isset($data[$this->prefix.$options['name']]) ? $data[$this->prefix.$options['name']] : $options['def_value'];
                }

                //global?
                if($options['global'])
                    continue;

                switch($options['type'])
                {
                    //multiselect
                    case 'm' :
                        //grid
                        if($type == 'grid')
                        {
                            $return[] = $this->grid_dropdown_row(
                                $options['label'],
                                $this->prefix.$options['name'].'[]',
                                $options['options'],
                                $value,
                                true
                            );
                        }

                        //normal
                        else
                        {
                            $return[] = array(
                                $options['label'],
                                form_multiselect(
                                    $this->prefix.$options['name'],
                                    $options['options'],
                                    $value
                                )
                            );
                        }
                        break;

                    //select field
                    case 's' :
                        //grid
                        if($type == 'grid')
                        {
                            $return[] = $this->grid_dropdown_row(
                                $options['label'],
                                $this->prefix.$options['name'],
                                $options['options'],
                                $value
                            );
                        }

                        //normal
                        else
                        {
                            $return[] = array(
                                $options['label'],
                                form_dropdown(
                                    $this->prefix.$options['name'],
                                    $options['options'],
                                    $value
                                )
                            );
                        }

                        break;

                    //text field
                    default:
                    case 't' :
                        //grid
                        if($type == 'grid')
                        {
                            $return[] =  form_label($options['label']).NBS.NBS.NBS.
                                form_input(array(
                                    'name'  => $this->prefix.$options['name'],
                                    'value' => $value,
                                    'class' => ''
                                )).NBS.NBS.NBS;
                        }

                        //normal
                        else
                        {
                            $return[] = array(
                                $options['label'],
                                form_input(
                                    $this->prefix.$options['name'],
                                    $value,
                                    $matrix_input
                                )
                            );
                        }
                        break;
                }
            }
        }

        return $return;
    }

    // ----------------------------------------------------------------------
    // Save the settings for all types
    // ----------------------------------------------------------------------

    /**
     * save_settings (Default EE)
     *
     * @access public
     */
    function save_settings($data)
    {
        return $this->_save_settings($data);
    }

    // --------------------------------------------------------------------

    /**
     * save_settings (EE 2.7 GRID)
     *
     * @access public
     */
//    function grid_save_settings($data)
//    {
//        return $this->_save_settings($data, true);
//    }

    // --------------------------------------------------------------------

    /**
     * save_settings
     *
     * @access public
     */
    private function _save_settings($data, $use_data = false)
    {
        $return = array();

        if(!empty($this->fieldtype_settings))
        {
            foreach($this->fieldtype_settings as $field=>$options)
            {
                //global?
                if($options['global'])
                    continue;

                $return[$this->prefix.$options['name']] =  $use_data ? $data[$this->prefix.$options['name']] : ee()->input->post($this->prefix.$options['name']);
            }
        }

        return $return;
    }

    // ----------------------------------------------------------------------

    /**
     * install
     *
     * @access public
     */
    function install()
    {
        $return = array();

        if(!empty($this->fieldtype_settings))
        {
            foreach($this->fieldtype_settings as $field=>$options)
            {
                if(isset($options['global']) && $options['global'])
                {
                    $return[$this->prefix.$options['name']] = $options['def_value'];
                }
            }
        }

        return $return;
    }

    // ----------------------------------------------------------------------

    /**
     * Update
     */
    function update()
    {
        return TRUE;
    }

    // ----------------------------------------------------------------------

    /**
     * display_global_settings
     *
     * @access public
     */
    function display_global_settings()
    {
        $data = array_merge($this->settings, $_POST);

        $form = '';
        foreach($this->fieldtype_settings as $field=>$options)
        {
            //global?
            if(!$options['global'])
                continue;

            switch($options['type'])
            {
                //multiselect
                case 'm' :
                    $form .= form_label($options['label'], $options['label']).NBS.form_multiselect($this->prefix.$options['name'], $options['options'], (isset($data[$this->prefix.$options['name']]) ? $data[$this->prefix.$options['name']] : $options['def_value'] )).NBS.NBS.NBS.' ';
                    break;

                //select field
                case 's' :
                    $form .= form_label($options['label'], $options['label']).NBS.form_dropdown($this->prefix.$options['name'], $options['options'], (isset($data[$this->prefix.$options['name']]) ? $data[$this->prefix.$options['name']] : $options['def_value'] )).NBS.NBS.NBS.' ';
                    break;

                //text field
                default:
                case 't' :
                    $form .= form_label($options['label'], $options['label']).NBS.form_input($this->prefix.$options['name'], (isset($data[$this->prefix.$options['name']]) ? $data[$this->prefix.$options['name']] : $options['def_value'] )).NBS.NBS.NBS.' ';
                    break;
            }
        }

        return $form;
    }

    // ----------------------------------------------------------------------

    /**
     * save_global_settings
     *
     * @access public
     */
    function save_global_settings()
    {
        return array_merge($this->settings, $_POST);
    }

    // ----------------------------------------------------------------------

    /**
     * accepts_content_type (GRID EE 2.7)
     *
     * @access public
     */
    public function accepts_content_type($name)
    {
        return ($name == 'channel');
        //return true;
    }

    // --------------------------------------------------------------------



    // ---------------------------------------------------------------------- 
    // Custom 
    // ----------------------------------------------------------------------

    // ----------------------------------------------------------------------

    /**
     * format_data
     *
     * @access private
     * @param $data
     * @return array
     */
    private function format_data( $data ) {
        $return = array();

        if(is_string($data))
        {
            $data = array_filter(explode('|', $data));

            if(is_array($data) && !empty($data))
            {
                foreach($data as $key => $val)
                {
                    $keys = explode('=', $val);

                    $return[$keys[0]] = urldecode($keys[1]);
                }
            }

            //merge with defaults
            $return = array_merge(array(
                'entry_id' => '',
                'custom' => '',
            ), $return);

            return $return;
        }

        return $data;

    }

    //------------------------------------------------------

    /**
     * @param int $entry_id
     * @return array|bool|string
     */
    private function get_site_pages($entry_id = 0)
    {
        //load structure data
        require_once PATH_THIRD.'structure/config.php';
        require_once PATH_THIRD.'structure/helper.php';
        require_once PATH_THIRD.'structure/sql.structure.php';
        require_once PATH_THIRD.'structure/mod.structure.php';

        //get the sql object
        $sql = new Sql_structure();

        if ( ! $sql->module_is_installed())
           return false;

        //get the site pages
        $site_pages = $sql->get_site_pages();

        if($entry_id != 0)
        {
            return isset($site_pages['uris'][$entry_id]) ? $site_pages['uris'][$entry_id] : '';
        }

        return $site_pages;
    }

    //------------------------------------------------------

    /**
     * @param $site_pages
     * @return array
     */
    private function add_zero($site_pages)
    {
        $return = array(0 => '');
        foreach($site_pages as $key => $val)
        {
            $return[$key] = $val;
        }

        return $return;
    }
}
